/*
 * $Id: fukuroengine.c,v 1.1.1.1 2002/10/19 13:07:02 kenta Exp $
 *
 * Copyright 2002 Kenta Cho. All rights reserved.
 */

/**
 * Fukuro engine.
 *
 * @version $Revision: 1.1.1.1 $
 */
#include <piece.h>
//#include "piece2sdl.h"

#include "fukuro.h"
#include "degutil.h"
#include "fukuroengine.h"
#include "hero.h"
#include "media.h"

Vagabond vagabond[VAGABOND_MAX];
Hero hero;

static int vgbApp[][4] = {
  {0, 0, -1, -1}, {FIELD_WIDTH-1, 0, 1, -1}, 
  {0, FIELD_HEIGHT-1, -1, 1}, {FIELD_WIDTH-1, FIELD_HEIGHT-1, 1, 1},
  {FIELD_WIDTH/2, 0, 0, -1},
};

static int vgbNum = 0;
static int cnt;

void addVagabond() {
  int p, hp;
  Vagabond *vgb;
  if ( vgbNum >= VAGABOND_MAX ) return;
  vgb = &(vagabond[vgbNum]);
  p = randN(4);
  if ( hero.x < FIELD_WIDTH/2 ) hp = 0;
  else hp = 1;
  if ( hero.x > FIELD_HEIGHT/2 ) hp += 2;

  if ( hp == p ) {
    p += randN(3); p &= 3;
  }
  if ( vgbNum == 0 ) p = 4;
  vgb->x = vgbApp[p][0] + randN(FIELD_WIDTH/5)*vgbApp[p][2];
  vgb->y = vgbApp[p][1] + randN(FIELD_HEIGHT/5)*vgbApp[p][3];
  vgb->mx = vgb->my = 0;
  vgb->walkCnt = 0;
  vgb->punchCnt = 0;
  vgb->blowCnt = 0;
  vgbNum++;
}

void initFukuro() {
  int i;
  Vagabond *vgb;
  for ( i=0 ; i<VAGABOND_MAX ; i++ ) {
    vgb = &(vagabond[i]);
    vgb->x = NOT_EXIST;
  }
  vgbNum = 0;
  cnt = 0;
  for ( i=0 ; i<4 ; i++ ) addVagabond();
}

#define SEPARATION_SPEED 128
#define ALIGNMENT_SPEED 256
#define COHESION_SPEED 920

#define SEPARATION_DEGREE_THRESHOLD 64
static int cohesionDegreeThreshold[] = {160, 80, 360};

#define PUNCH_DISTANCE 5120
#define PUNCH_INTERVAL 32

void moveFukuro() {
  int i, j;
  Vagabond *vgb, *rvgb;
  int ox, oy;
  long ar = 0;
  int vn = 0;
  int vx, vy;
  int si, sad, sdt;
  int od;
  int cdt = cohesionDegreeThreshold[status];

  for ( i=0 ; i<VAGABOND_MAX ; i++ ) {
    if ( vagabond[i].x == NOT_EXIST ) continue;
    vgb = &(vagabond[i]);
    ox = hero.x-vgb->x; oy = hero.y-vgb->y;
    vgb->d = getDeg(ox, oy);
    vgb->r = getDistance(ox, oy);
    ar += vgb->r;
    vn++;
  }
  if ( vn == 0 ) return;
  ar /= vn;
  for ( i=0 ; i<VAGABOND_MAX ; i++ ) {
    if ( vagabond[i].x == NOT_EXIST ) continue;
    vgb = &(vagabond[i]);

    if ( vgb->blowCnt > 0 ) {
      // Blowed.
      vgb->blowCnt--;
      vgb->mx = vgb->bmx; vgb->my = vgb->bmy;
      vgb->bmx -= (vgb->bmx>>3);
      vgb->bmy -= (vgb->bmy>>3);
    } else {
      // Separation.
      si = -1; sdt = SEPARATION_DEGREE_THRESHOLD;
      for ( j=0 ; j<VAGABOND_MAX ; j++ ) {
	if ( vagabond[j].x == NOT_EXIST || i == j ) continue;
	rvgb = &(vagabond[j]);
	sad = absN(rvgb->d - vgb->d);
	if ( sad < sdt ) {
	  si = j; sdt = sad;
	}
      }
      if ( si == -1 ) {
	vx = vy = 0;
      } else {
	od = getDeg(vgb->x-vagabond[si].x, vgb->y-vagabond[si].y);
	vx =  ((sctbl[od]*SEPARATION_SPEED)>>8);
	vy = -((sctbl[od+DIV/4]*SEPARATION_SPEED)>>8);
      }
	
      // Alignment.
      od = vgb->d;
      if ( ar > vgb->r ) {
	vx -= ((sctbl[od]*ALIGNMENT_SPEED)>>8);
	vy += ((sctbl[od+DIV/4]*ALIGNMENT_SPEED)>>8);
      } else {
	vx += ((sctbl[od]*ALIGNMENT_SPEED)>>8);
	vy -= ((sctbl[od+DIV/4]*ALIGNMENT_SPEED)>>8);
      }

      // Cohesion.
      od = vgb->d - hero.d;
      while ( od >= 1024 ) od -= 1024;
      while ( od < 0 ) od += 1024;
      if ( od < cdt || od > 1024-cdt ) {
	od = vgb->d;
	vx += ((sctbl[od]*COHESION_SPEED)>>8);
	vy -= ((sctbl[od+DIV/4]*COHESION_SPEED)>>8);
      }

      vgb->mx += (vx-vgb->mx)>>4;
      vgb->my += (vy-vgb->my)>>4;
      vgb->walkCnt += (absN(vgb->mx)+absN(vgb->my))>>2;

      if ( vgb->punchCnt > -PUNCH_INTERVAL ) {
	vgb->punchCnt--;
	if ( vgb->punchCnt == 6 ) {
	  damageHero(vgb->d);
	}
      } else {
	if ( vgb->r < PUNCH_DISTANCE && status != DEMO ) {
	  playSeNow(SE_PUNCH2, 3, 0);
	  vgb->punchCnt = 8;
	}
      }
    }

    vgb->x += vgb->mx;
    vgb->y += vgb->my;
  }
  cnt++; if ( (cnt&63) == 0 ) addVagabond();
}

#define PUNCH_COLLISION 6400

int checkPunchHitVagabond(int x, int y) {
  int i;
  Vagabond *vgb;
  int d, mind = PUNCH_COLLISION, minIdx = -1;
  for ( i=0 ; i<VAGABOND_MAX ; i++ ) {
    if ( vagabond[i].x == NOT_EXIST ) continue;
    vgb = &(vagabond[i]);
    d = getDistance(vgb->x-x, vgb->y-y);
    if ( d < mind ) {
      mind = d; minIdx = i;
    }
  }
  if ( mind < PUNCH_COLLISION ) {
    int od, sp;
    vgb = &(vagabond[minIdx]);
    od = getDeg(vgb->x-hero.x, vgb->y-hero.y);
    od -= hero.d;
    if ( od >= 512 ) od -= 1024;
    else if ( od < -512 ) od += 1024;
    sp = sctbl[((od/2)&(DIV-1))+DIV/4];
    od *= 2;
    od = (hero.d+od)&(DIV-1);
    vgb->bmx =  (sctbl[od]*sp)>>4;
    vgb->bmy = -(sctbl[od+DIV/4]*sp)>>4;
    vgb->blowCnt = 32;
    vgb->punchCnt = 0;
    return 1;
  }
  return 0;
}

#define FOOT_POS 7
#define FOOT_ROOT_POS 4
#define HEAD_ROOT_POS 2
#define HEAD_POS 5
#define HAND_POS 2

void drawFukuro() {
  int i;
  Vagabond *vgb;
  int x, y;
  int ox, oy;
  int hx, hy;
  int c;
  for ( i=0 ; i<VAGABOND_MAX ; i++ ) {
    if ( vagabond[i].x == NOT_EXIST ) continue;
    vgb = &(vagabond[i]);
    x = vgb->x*FIELD_DOT_WIDTH/FIELD_WIDTH;
    y = vgb->y*FIELD_DOT_HEIGHT/FIELD_HEIGHT;
    // Body.
    drawLine(3, x, y-HEAD_ROOT_POS, x, y+FOOT_ROOT_POS);

    ox = sctbl[vgb->d+DIV/4]>>6;
    oy = sctbl[vgb->d]>>7;
    // Head.
    drawLine(3, x+ox, y+oy-HEAD_POS, x-ox, y-oy-HEAD_POS);
    drawLine(3, x, y-HEAD_ROOT_POS, x+ox, y+oy-HEAD_POS);
    drawLine(3, x, y-HEAD_ROOT_POS, x-ox, y-oy-HEAD_POS);

    // Hand.
    if ( vgb->blowCnt <=0 && vgb->punchCnt > 0 ) {
      c = sctbl[((vgb->punchCnt)*32+48)&(DIV-1)];
      hx = ( sctbl[vgb->d]*c)>>12;
      hy = (-sctbl[vgb->d+DIV/4]*c)>>12;
    } else {
      hx = hy = 0;
    }
    drawLine(3, x, y-HAND_POS, x+ox+hx, y+oy+hy+HAND_POS);
    drawLine(3, x, y-HAND_POS, x-(hx>>2)-ox, y-(hy>>2)-oy+HAND_POS);

    if ( vgb->blowCnt <= 0 ) {
      c = ((vgb->walkCnt>>8)&31)*32;
      ox += (vgb->mx*sctbl[c])>>15;
      oy += (vgb->my*sctbl[c])>>15;
    } else {
      ox = oy = 0;
    }
    // Foot.
    drawLine(3, x, y+FOOT_ROOT_POS, x+ox, y+oy+FOOT_POS);
    drawLine(3, x, y+FOOT_ROOT_POS, x-ox, y-oy+FOOT_POS);
  }
}
