/*
 * $Id: hero.c,v 1.1.1.1 2002/10/19 13:07:02 kenta Exp $
 *
 * Copyright 2002 Kenta Cho. All rights reserved.
 */

/**
 * Move my hero.
 *
 * @version $Revision: 1.1.1.1 $
 */
#include <piece.h>
//#include "piece2sdl.h"

#include "hero.h"
#include "fukuro.h"
#include "fukuroengine.h"
#include "degutil.h"
#include "media.h"

int score;

void initHero() {
  hero.x = FIELD_WIDTH/2; hero.y = FIELD_HEIGHT/2;
  hero.d = 512;
  hero.walkCnt = 0;
  hero.mx = hero.my = 0;
  hero.punchCnt = 0;
  hero.damage = 0;
  hero.bmx = hero.bmy = 0;
  score = 0;
}

void setHeroRandomPos() {
  hero.x = randN(FIELD_WIDTH>>4)<<4;
  hero.y = randN(FIELD_HEIGHT>>4)<<4;
  hero.d = randN(DIV);
}

#define HERO_SPEED 512

#define FOOT_POS 8
#define FOOT_ROOT_POS 4
#define HEAD_ROOT_POS 2
#define HEAD_POS 6
#define HAND_POS 2

void moveHero(int pad) {
  int ad = -1;
  if ( pad & PAD_UP ) {
    ad = 0;
  } else if ( pad & PAD_DN ) {
    ad = 512;
  }
  if ( pad & PAD_LF ) {
    if ( ad == -1 ) {
      ad = 768;
    } else {
      ad = -(ad-256)/2;
      ad += 768;
    }
  } else if ( pad & PAD_RI ) {
    if ( ad == -1 ) {
      ad = 256;
    } else {
      ad = (ad-256)/2;
      ad += 256;
    }
  }
  if ( ad != -1 ) {
    ad -= hero.d;
    if ( ad > 512 ) ad -= 1024;
    if ( ad < -512 ) ad += 1024;
    hero.d += ad>>1;
    if ( hero.d < 0 ) hero.d += 1024;
    if ( hero.d >= 1024 ) hero.d -= 1024;
    hero.mx =  (sctbl[hero.d]*HERO_SPEED)>>8;
    hero.my = -(sctbl[hero.d+DIV/4]*HERO_SPEED)>>8;
    hero.x += hero.mx; 
    hero.y += hero.my;
    hero.walkCnt++;
  } else {
    if ( hero.punchCnt == 0 ) {
      hero.damage -= 3;
      if ( hero.damage <= 0 ) hero.damage = 0;
    }
  }

  if ( hero.punchCnt <= 0 ) {
    if ( pad & (TRG_A | TRG_B) ) {
      playSeNow(SE_PUNCH1, 1, 0);
      hero.punchCnt = 8;
    }
  } else {
    hero.punchCnt--;
    if ( hero.punchCnt == 7 ) {
      int x, y, c;
      x = hero.x*FIELD_DOT_WIDTH/FIELD_WIDTH;
      y = hero.y*FIELD_DOT_HEIGHT/FIELD_HEIGHT;
      x += sctbl[hero.d+DIV/4]>>6;
      y += sctbl[hero.d]>>7;
      c = sctbl[((hero.punchCnt)*32+48)&(DIV-1)];
      x += ( sctbl[hero.d]*c)>>13;
      y += (-sctbl[hero.d+DIV/4]*c)>>13;
      y += HAND_POS;
      x = x*FIELD_WIDTH/FIELD_DOT_WIDTH;
      y = y*FIELD_HEIGHT/FIELD_DOT_HEIGHT;
      if ( checkPunchHitVagabond(x, y) ) {
	playSeNow(SE_HIT1, 2, 0);
	score++;
      }
    }
  }
  hero.x += hero.bmx;
  hero.y += hero.bmy;
  hero.bmx -= (hero.bmx>>2);
  hero.bmy -= (hero.bmy>>2);

  if ( hero.x < 0 ) {
    hero.x = 0;
  } else if ( hero.x >= FIELD_WIDTH ) {
    hero.x = FIELD_WIDTH-1;
  }
  if ( hero.y < 0 ) {
    hero.y = 0;
  } else if ( hero.y >= FIELD_HEIGHT ) {
    hero.y = FIELD_HEIGHT-1;
  }
}

void drawHero() {
  int x, y;
  int ox, oy;
  int hx, hy;
  int c;
  x = hero.x*FIELD_DOT_WIDTH/FIELD_WIDTH;
  y = hero.y*FIELD_DOT_HEIGHT/FIELD_HEIGHT;
  // Body.
  drawLine(3, x, y-HEAD_ROOT_POS, x, y+FOOT_ROOT_POS);

  ox = sctbl[hero.d+DIV/4]>>6;
  oy = sctbl[hero.d]>>7;
  // Head.
  drawLine(3, x+ox, y+oy-HEAD_ROOT_POS, x-ox, y-oy-HEAD_ROOT_POS);
  drawLine(3, x+ox, y+oy-HEAD_POS, x-ox, y-oy-HEAD_POS);
  drawLine(3, x+ox, y+oy-HEAD_ROOT_POS, x+ox, y+oy-HEAD_POS);
  drawLine(3, x-ox, y-oy-HEAD_ROOT_POS, x-ox, y-oy-HEAD_POS);

  // Hand.
  if ( hero.punchCnt > 0 ) {
    c = sctbl[((hero.punchCnt)*32+48)&(DIV-1)];
    hx = ( sctbl[hero.d]*c)>>12;
    hy = (-sctbl[hero.d+DIV/4]*c)>>12;
  } else {
    hx = hy = 0;
  }
  drawLine(3, x, y-HAND_POS, x+ox+hx, y+oy+hy+HAND_POS);
  drawLine(3, x, y-HAND_POS, x-(hx>>2)-ox, y-(hy>>2)-oy+HAND_POS);

  c = sctbl[(hero.walkCnt&31)*32];
  ox += (hero.mx*c)>>14;
  oy += (hero.my*c)>>14;
  // Foot.
  drawLine(3, x, y+FOOT_ROOT_POS, x+ox, y+oy+FOOT_POS);
  drawLine(3, x, y+FOOT_ROOT_POS, x-ox, y-oy+FOOT_POS);

  // Damage meter.
  if ( hero.damage < 1000 ) {
    pceLCDPaint(1, 14, 82, (1000-hero.damage)/10, 3);
  }
}

void drawDeadHero(int cnt) {
  int x, y;
  int ox, oy;
  int dy1, dy2;

  x = hero.x*FIELD_DOT_WIDTH/FIELD_WIDTH;
  y = hero.y*FIELD_DOT_HEIGHT/FIELD_HEIGHT;
  dy1 = -((HEAD_ROOT_POS+FOOT_ROOT_POS)*sctbl[cnt*10+DIV/4])>>8;
  dy1 += y+FOOT_ROOT_POS;
  // Body.
  drawLine(3, x, y+FOOT_ROOT_POS, x, dy1);

  ox = sctbl[hero.d+DIV/4]>>6;
  oy = sctbl[hero.d]>>7;
  dy2 = -((HEAD_POS+FOOT_ROOT_POS)*sctbl[cnt*10+DIV/4])>>8;
  dy2 += y+FOOT_ROOT_POS;
  // Head.
  drawLine(3, x+ox, dy1+oy, x-ox, dy1-oy);
  drawLine(3, x+ox, dy2+oy, x-ox, dy2-oy);
  drawLine(3, x+ox, dy1+oy, x+ox, dy2+oy);
  drawLine(3, x-ox, dy1-oy, x-ox, dy2-oy);
}

static int scPos[][4] = {
  {1,1,10,0},{6,1,5,10},{6,6,10,5},{2,6,2,10},{0,11,11,10}
};

void drawScore() {
  int s = score;
  int x = 4, y = 0;
  int i;
  for ( ; ; ) {
    for ( i=0 ; i<5 ; i++, s-- ) {
      if ( s <= 0 ) return;
      drawLine(2, scPos[i][0]+x, scPos[i][1]+y, scPos[i][2]+x, scPos[i][3]+y); 
    }
    x += 12;
    if ( x >= SCREEN_WIDTH-11 ) {
      x = 4; y += 12;
    }
  }
}

void damageHero(int d) {
  if ( status != IN_GAME ) return;
  playSeNow(SE_HIT2, 3, 0);
  hero.damage += 250;
  if ( hero.damage >= 1000 ) {
    initGameover();
  } else {
    hero.bmx += sctbl[d]<<3;
    hero.bmy -= sctbl[d+DIV/4]<<3;
  }
}
